#!/usr/bin/env python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
from __future__ import (unicode_literals, division, absolute_import,
                        print_function)

__license__   = 'GPL v3'
__copyright__ = '2011, DrMerry based on Pr.BarnArt, based on the Barnes work by Grant Drake'
__docformat__ = 'restructuredtext en'

import socket, re, datetime
from collections import OrderedDict
from threading import Thread
from lxml.html import fromstring, tostring



from calibre.ebooks.metadata.book.base import Metadata
from calibre.library.comments import sanitize_comments_html
from calibre.utils.cleantext import clean_ascii_chars


import calibre_plugins.catawiki.config as cfg

class Worker(Thread): # Get  details

    '''
    Get book details from catawiki.nl book page in a separate thread
    related to catawiki
    '''
    name                    = 'Worker'
    description             = _('Get book details from catawiki.nl book page in a separate thread')
    author                  = 'DrMerry based on Pr. BarnArt'
    version                 = (0, 0, 1)
    minimum_calibre_version = (0, 8, 0)


    def __init__(self, combo, result_queue, browser, log, relevance, plugin, timeout=20):       
        Thread.__init__(self)
        self.daemon = True
        #distract from combo
        self.url = combo[0]
        self.rating = combo[1]
        self.result_queue =  result_queue
        self.log, self.timeout = log, timeout
        self.relevance, self.plugin = relevance, plugin
        self.browser = browser.clone_browser()
        self.cover_url = self.catawiki_nl_id = self.isbn = None

    def run(self):
        try:
            self.get_details()
        except:
            self.log.exception('get_details failed for url: %r'%self.url)

    def get_details(self):
        try:
            self.log.info('catawiki url: %r'%self.url)
            raw = self.browser.open_novisit(self.url, timeout=self.timeout).read().strip()
        except Exception as e:
            if callable(getattr(e, 'getcode', None)) and \
                    e.getcode() == 404:
                self.log.error('URL malformed: %r'%self.url)
                return
            attr = getattr(e, 'args', [None])
            attr = attr if attr else [None]
            if isinstance(attr[0], socket.timeout):
                msg = 'catawiki.nl timed out. Try again later.'
                self.log.error(msg)
            else:
                msg = 'Failed to make details query: %r'%self.url
                self.log.exception(msg)
            return

        raw = raw.decode('utf-8', errors='replace')
 

        if '<title>404 - ' in raw:
            self.log.error('URL malformed: %r'%self.url)
            return

        
        
        try:
            root = fromstring(clean_ascii_chars(raw))            
        except:
            msg = 'Failed to parse catawiki.nl details page: %r'%self.url
            self.log.exception(msg)
            return

        self.parse_details(root,raw)

    def parse_details(self, root,raw):      
        try:
            catawiki_nl_id = self.parse_catawiki_nl_id(self.url)
            
        except:
            self.log.exception('Error parsing catawiki.nl id for url: %r'%self.url)
            catawiki_nl_id = None
        try:            
            title = self.parse_title(raw)
            series = None
            series_index = None
        except:
            self.log.exception('Error parsing title for url: %r'%self.url)
            title = None
        try:
            authors = self.parse_authors(raw)
        except:
            self.log.exception('Error parsing authors for url: %r'%self.url)
            authors = []            
        if not title or not authors or not catawiki_nl_id:
            self.log.error('Could not find title/authors/catawiki_nl_id for %r'%self.url)
            self.log.error('catawiki_nl_id: %r Title: %r Authors: %r'%(catawiki_nl_id, title,
                authors))
        
            return
        mi = Metadata(title, authors)
        
        if series:
            mi.series = series
            mi.series_index = series_index
        self.catawiki_nl_id = catawiki_nl_id
       
        try:
            isbn = self.parse_isbn(raw)
            if isbn:
                self.isbn = isbn
                mi.isbn = isbn
        except:
            self.log.exception('Error parsing ISBN for url: %r'%self.url)

        #do rating coversion from txt to integer
        rat=float(self.rating)
        rat = rat + 0.5
        rat=int(rat)
        mi.rating=rat
        
        
        try:
            mi.comments = self.parse_comments(raw)
        except:
            self.log.exception('Error parsing comments for url: %r'%self.url)

        try:
            self.cover_url = self.parse_cover(root)
        except:
            self.log.exception('Error parsing cover for url: %r'%self.url)
        mi.has_cover = bool(self.cover_url)

        try:
            mi.publisher = self.parse_publisher(raw)
        except:
            self.log.exception('Error parsing publisher for url: %r'%self.url)

        try:
            mi.pubdate = self.parse_published_date(raw)
        except:
            self.log.exception('Error parsing published date for url: %r'%self.url)

        mi.source_relevance = self.relevance

        if self.catawiki_nl_id:
            if self.isbn:
                self.plugin.cache_isbn_to_identifier(self.isbn, self.catawiki_nl_id)

        self.plugin.clean_downloaded_metadata(mi)
        self.result_queue.put(mi)
        
    def parse_catawiki_nl_id(self, url):
        return url
        #/catalogus/strips/series-helden/kuifje/24306-de-7-kristallen-bollen
        #return re.search('catawiki.nl/(.*/.*/\d+)', url).groups(0)[0]

    def parse_title(self,raw):
        #location = self.url
        findstart=raw.find('<h1>')
        findend=raw.find('</h1>')
        title=raw[findstart+4:findend]
        #findend=title.find('/')
        #title=title[0:findend]
        title=title.replace('-',' ')       
        return title
    
    def parse_authors(self, raw):
        authors = []
        findtext='<th scope="row">Tekenaar(s):</th>'
        findstart = raw.find(findtext)
        prodtxt=raw[findstart+len(findtext):findstart+200]
        if findstart>0:
            authors.append(self.parse_authors_part2(' id="area_ref">', prodtxt, '</a>'))
            valid_contrib = None
        findtext='<th scope="row">Scenarist(en):</th>'
        findstart = raw.find(findtext)
        prodtxt=raw[findstart+len(findtext):findstart+200]
        if findstart>0:
            authors.append(self.parse_authors_part2(' id="area_ref">', prodtxt, '</a>'))
            valid_contrib = None
        findtext='<th scope="row">Auteur:</th>'
        findstart = raw.find(findtext)
        prodtxt=raw[findstart+len(findtext):findstart+200]
        if findstart>0:
            authors.append(self.parse_authors_part2(' id="area_ref">', prodtxt, '</a>'))
            valid_contrib = None
        return authors
            
    def parse_authors_part2(self, findstartstring, prodtxt, findendstring):
        findstart=prodtxt.find(findstartstring)
        author=prodtxt[findstart+len(findstartstring):len(prodtxt)]
        findend=author.find(findendstring)
        author=author[0:findend]
        return author.strip()
            
   
    def parse_isbn(self, raw): 
        findstart = raw.find('<th scope="row">ISBN:</th>')
        prodtxt=raw[findstart:findstart+100]
        findstart=prodtxt.find('<td>')
        if findstart>0:
            findend=prodtxt.find('</td>')
            isbn=prodtxt[findstart+4:findend]
            return isbn
            
    def parse_rating(self, root):
        return 0
        '''rating_node = root.xpath('//div[@class="w-box wgt-product-ratings"]/a/div/@class')
        if rating_node:
            # After the rewrite B&N no longer put the actual values of the rating in the web page
            # Instead they put words like "four half" for 4.5 and "four" for "4" in the style
            # <div class="product-rating four half">
            rating_class = rating_node[0]
            match = re.search('product-rating (.+)', rating_class)
            if match:
                rating_text = match.groups(0)[0]
                rating_parts = rating_text.split(' ')
                rating_values = ['zero','one','two','three','four','five']
                rating_value = float(rating_values.index(rating_parts[0]))
                if len(rating_parts) > 1:
                    rating_value += 0.5
                return rating_value
        else:
            # Try the textbook page rating lookup
            # <span class="avg-4h section_updateRating">
            rating_node = root.xpath('//span[contains(@class,"section_updateRating")]/@class')
            if rating_node:
                rating_text = rating_node[0][4:6]
                rating_value = float(rating_text[0])
                if rating_text[1] == 'h':
                    rating_value += 0.5
                return rating_value'''


        
    def parse_publisher(self, raw):
        publisher = None        
        findstart = raw.find('<th scope="row">Uitgeverij:</th>')
        prodtxt=raw[findstart:findstart+100]
        findstart=prodtxt.find(' id="area_ref">')
        if findstart>0:
            findend=prodtxt.find('</a>')
            publisher=prodtxt[findstart+4:findend]
            return publisher

    def parse_published_date(self, raw):
        pub_date = None        
        findstart = raw.find('<th scope="row">Jaar:</th>')
        prodtxt=raw[findstart:findstart+100]
        findstart=prodtxt.find('<td>')
        if findstart>0:
            findend=prodtxt.find('</td>')
            pub_date=prodtxt[findstart+4:findend]
            #return pub_date
        if not(pub_date is None):
            return  self._convert_date_text(pub_date)
            

            
    def _convert_date_text(self, date_text):
        # Note that the date text could be "2003", "december 2003" or "December 10th 2003"
        year = int(date_text[-4:])
        month = 1
        day = 1
        if len(date_text) > 4:
            text_parts = date_text[:len(date_text)-5].partition(' ')
            month_name = text_parts[0]
            # Need to convert the month name into a numeric value
            # For now I am "assuming" the Goodreads website only displays in English
            # If it doesn't will just fallback to assuming January
            month_dict = {"januari":1, "februari":2, "maart":3, "april":4, "mei":5, "juni":6,
                "juli":7, "augustus":8, "september":9, "oktober":10, "november":11, "december":12}
            month = month_dict.get(month_name, 1)
            if len(text_parts[2]) > 0:
                day = int(re.match('([0-9]+)', text_parts[2]).groups(0)[0])
        from calibre.utils.date import utc_tz
        return datetime.datetime(year, month, day, tzinfo=utc_tz)

    def parse_comments(self, raw):
        comments = 'Catawiki comment<hr>'
        findstart = raw.find('<th scope="row">Bijzonderheden:</th>')
        prodtxt=raw[findstart:findstart+1000]
        findstart=prodtxt.find('<td>')
        if findstart>0:
            findend=prodtxt.find('<script')
            comments=comments+prodtxt[findstart+4:findend]
            return comments
            

    def parse_cover(self, root):
        result_node=root.xpath('//div [@id="images"]/a [@class="thickbox image image_1"]/@href')    
        if result_node:
            img_url = result_node[0]
            #img_url = img_url.replace('thumb','large')
            self.plugin.cache_identifier_to_cover_url(self.catawiki_nl_id, img_url)
            return img_url
        #geen  kijkexemplaar
        # result_node=root.xpath('//div [@class="product_image"]/img/@src')
        # if result_node:
            # img_url=result_node[0]
            # self.plugin.cache_identifier_to_cover_url(self.catawiki_nl_id, img_url)
            # return img_url
        
             
            
